import {actionPromise, gql,origUrl} from '../App';


const actionUpload = (files) =>
	async (dispatch) => {
		const formData = new FormData();
		formData.append('photo', files);
		await dispatch(actionUploadFile(formData));
	}


  const actionUploadFile = formData =>
	actionPromise('upload',
		fetch(`${origUrl}upload`, {
			method: "POST",
			headers: localStorage.authToken ? {Authorization: 'Bearer ' + localStorage.authToken} : {},
			body: formData
		}).then(res => res.json())
	);


const actionlistCount = () => 
  actionPromise('listCount', gql(`query listCount{
    AdCount(query:"[{}]") }`))


const actionOnelist = (_id) => 
actionPromise('onelist', gql(`query onelist($q: String!) {
    AdFindOne(query:$q) {
      price _id createdAt owner {
                  _id login nick avatar {
                    _id url
                  }
      }
      title description images {
        _id url
      }
      comments {
                  _id owner {
                    _id login nick avatar {
                      _id url
                    }
                  }
                  createdAt
                  text
                  answerTo {
                    _id
                  }
      }
    }
  }`,{q: JSON.stringify([{_id}])}
)) 


const actionLogin = (login, password) =>
actionPromise('login', gql(`query Login($login: String!, $password: String!) {
     login(login: $login, password: $password)
   }`, {login, password}));


const actionRegister = (login, password) => 
actionPromise('reg', gql(`mutation Reg($login: String!, $password: String!) {
     createUser(login: $login, password: $password) {
       _id
     }
   }`, 
{"login" : login,
"password": password}
))

const actionUpdateMe = (user) =>
    actionPromise('profile', gql(`mutation upsertProfile($user:UserInput) {
                                        UserUpsert(user:$user) {
                                            _id login nick
                                        }
                                    }`, {user}))



const actioCreateAd = (ad) =>
actionPromise('createAd', gql(`mutation userAd($ad:AdInput) {
  AdUpsert(ad: $ad) {
      _id title images{
          _id url
      }
  }
}`, {ad}))


const actionOneUser = (_id) =>
actionPromise('user', gql(`query userOne($qq: String!){
  UserFindOne(query:$qq){
    _id
    login
   phones
   addresses
   nick 
   incomings{
    _id text createdAt  owner{
      _id login
    }
   }
   avatar{
      url _id
    }
  }
}`, {qq: JSON.stringify([{_id}])}))



const actionAdComment = (comment) =>
actionPromise('adComment', gql(`mutation adComment($comment:CommentInput) {
  CommentUpsert(comment: $comment) {
    _id
    text owner {
      _id login nick 
    }
  }
}`, {comment}))


const actionAdMessage = (message) =>
actionPromise('admessage', gql(`mutation admessage($message:MessageInput) {
  MessageUpsert(message: $message) {
    _id
    text
    createdAt
    to{
      _id login
    }
    owner{
      _id
      login
      nick
    }
  }
}`, {message}))


const actionAllProductsUser = (_id) =>
actionPromise("productsUser", gql(`query productsUser($userId: String) {
                AdFind(query: $userId) {
                  price _id owner {
                                      _id login nick avatar {
                                        _id url
                                      }
                                    }
                                    title description images {
                                      _id url
                                    }
                }
              }`, {userId: JSON.stringify([{___owner: _id}])}));


const actionMyUserMessage = (_id) => 
actionPromise('allMeMessage', gql(`query allMeMessage($userId: String){
  MessageFind(query: $userId){
    _id
  text
  createdAt
  to{
    _id login avatar {_id url}
  }
  owner{
    _id
    login
    nick
  }
  }
}`, {userId: JSON.stringify([{___owner: _id}])}));


const actionForMeMessage = (_id) =>
actionPromise('allForMeMessage', gql(`query forMeMessage($qq: String!){
  UserFindOne(query:$qq){
   incomings{
    _id text createdAt  owner{
      _id login avatar {_id url}
    }
   }
  }
}`, {qq: JSON.stringify([{_id}])}))


const actionList = (skip = 0, limit=15) =>
    actionPromise('list', gql(`query List($query: String) {
					  AdFind(query:$query) {
					    price _id owner {
                          _id login nick avatar {
                            _id url
                          }
					    }
					    title description createdAt images {
					      _id url
					    }
					  }
					}`, {query: JSON.stringify([
                            {
                                $or: [{}]
                            },
                            {
                              sort: [{_id: -1}],
                                limit: [limit],
                                skip: [skip]
                            }
                        ])}
    ));


const actionListSearch = (searchText = '', skip = 0, limit=15) =>
actionPromise('list', gql(`query searchList($query: String) {
        AdFind(query:$query) {
          price _id owner {
                      _id login nick avatar {
                        _id url
                      }
          }
          title description createdAt images {
            _id url
          }
        }
      }`, {query: JSON.stringify([
                        {
                            $or: [{title: `/${searchText}/`}, {description: `/${searchText}/`}]
                        },
                        {
                          sort: [{_id: -1}],
                            limit: [limit],
                            skip: [skip]
                        }
                    ])}
));



const actionSearchCount = (searchText = '') =>
actionPromise('listCount', gql(`query searchCount($query: String) {
        AdCount(query:$query)}`, 
        {query: JSON.stringify([
                        {
                            $or: [{title: `/${searchText}/`}, {description: `/${searchText}/`}]
                        }
                    ])}
));


export {
    actionAllProductsUser,
    actionMyUserMessage,
    actionForMeMessage,
    actionList,
    actionListSearch,
    actionSearchCount,
    actionlistCount,
    actionLogin,
    actionUpload,
    actionOneUser,
    actionAdMessage, 
    actionOnelist,
    actionAdComment, 
    actioCreateAd,
    actionRegister, 
    actionUpdateMe,
  } 